require_dependency "user_mgmt/application_controller"
require_dependency "user_mgmt/application_helper"

module UserMgmt
  class SessionsController < ApplicationController
    include ApplicationHelper
    
    # GET /user/log_in
    def new

    end

    # POST /user/log_in
    def create

        response = log_in *params[:user].values
      
        if response[:status] == "200"  
            flash[:success] = "User logged in!"
            cookies[:user_session_id] = { 
                value: response[:body]["session"],
                expires: 1.hour.from_now,
                domain: ".stewardly.ca"
            }
            redirect_to main_app.root_path
        else 
            flash[:error] = response[:body]
            redirect_to log_in_path      
        end

    end

    # GET 'auth/:provider/callback'
    # That is the controller method that is called after getting the information from an oauth strategy
    def oauth_email

        @uid = env['omniauth.auth']["uid"]
        @provider= env['omniauth.auth']["provider"]

        if @provider == "google_oauth2"
            @provider = "google"
        end
        
        if @provider == "windowslive"
            @provider = "windows_live"
        end

        @umid = current_user["umid"] if user_logged_in?

        response = log_in_oauth @uid, @provider
        
        # if user logged in succesfully, just redirect to main page,
        # otherwise it means the user doesn't exist yet, so continue 
        # normaly (user will be asked for his email).
        if response[:status] == "200"
            if @umid
                redirect_to main_app.root_path, flash: { error: "User already signed up for the app!" }
            else
                cookies[:user_session_id] = { 
                    value: response[:body]["session"],
                    expires: 1.hour.from_now,
                    domain: ".stewardly.ca"
                }
                redirect_to main_app.root_path, flash: { success: "User logged in!" }
            end
        elsif @umid
            add_strategy @uid, @provider, @umid, cookies[:user_session_id]
            redirect_to main_app.root_path, flash: { success: "Strategy added successfully!" }
        end

    end

    # POST '/oauth_log_in'
    def create_oauth

        response = sign_up_oauth params["uid"], params["provider"].to_sym, params["user"]["Email"] 

        #if for some reason, the user couldn't be created
        if response[:status] == "400"
            redirect_to sign_up_path, flash: { error: response[:body] }
        else
            response = log_in_oauth params["uid"], params["provider"].to_sym
            cookies[:user_session_id] = { 
                value: response[:body]["session"],
                expires: 1.hour.from_now,
                domain: ".stewardly.ca"
            }
            redirect_to main_app.root_path, flash: { success: "User logged in!" }
        end

    end

    # GET /user/log_out
    def destroy
        cookies.delete(:user_session_id, domain: ".stewardly.ca")
        redirect_to main_app.root_path, flash: { success: "Logged out!" }
    end

  end
end
